# Instagram Time ID converter

import datetime


def get_location() -> str:
	return input(
		'Enter the URL of the location (for example: https://www.instagram.com/explore/locations/95099702/mgm-grand-las-vegas/)')


def get_date() -> str:
	return input('Enter the date in this form: 1999/7/27... (leave no zeros in front of single digit months/days)')


def date_str_2_dateobj(date: str) -> datetime.datetime:
	d_list = date.split("/")
	return datetime.datetime(int(d_list[0]), int(d_list[1]), int(d_list[2]), 23, 59, 59)


def date_2_unix(date_obj: datetime.date) -> int:
	unixdate = date_obj - datetime.datetime(1970, 1, 1)
	mstime = int(unixdate.total_seconds() * 1000.0)
	insta_epoch = mstime - 1314220021300
	return insta_epoch


def binary_decimal_convert(bindec: tuple) -> int:
	#     print(bin(bindec[1]))
	if bindec[0]:
		return int(bin(bindec[1])[2:])
	else:
		return int(bindec[1], 2)


def binary_lengthen(binary: int) -> int:
	zeroes = 41 - len(str(binary))
	six_fourbit = ('0' * zeroes) + str(binary) + ('0' * 23)
	return six_fourbit


def run(date):
	#     location_url = get_location()
	unix_time = date_2_unix(date_str_2_dateobj(date))
	newbin = binary_decimal_convert((True, unix_time))
	longbin = binary_lengthen(newbin)
	final_num = binary_decimal_convert((False, longbin))
	return str(final_num)



import argparse
import codecs
import json
import logging
import sys
from pathlib import Path

import backoff
import requests
from tqdm import tqdm

from insta_graphql_scraper import dateToId


BASE_URL = "https://www.instagram.com/explore/locations/{location_id}/?__a=1&max_id={max_id}"
MAX_TRIES = 11


@backoff.on_exception(wait_gen=backoff.fibo, max_tries=MAX_TRIES,
                      exception=(requests.exceptions.HTTPError, requests.exceptions.ConnectionError))
def pull_json(location_name, end_cursor):
	URL = BASE_URL.format(location_id=METRO[location_name], max_id=end_cursor)
	logging.debug(URL)
	r = requests.get(url=URL)
	if r.status_code == 200:
		data = r.json()['graphql']['location']['edge_location_to_media']
		return data
	else:
		raise requests.exceptions.HTTPError


def save_jsonl(data, dst='./'):
	"""Saves the data to a jsonl file."""
	path = Path(dst)
	assert isinstance(data, list)
	if path.exists():
		f = codecs.open(dst, "ab", 'utf-32')
	else:
		path.parent.mkdir(parents=True, exist_ok=True)
		f = codecs.open(dst, 'wb', 'utf-32')
	f.writelines("%s\n" % json.dumps(s) for s in data)


def scrape(date1, date2, location, restore_cursor=None):
	try:
		maxid1 = dateToId.run(date1)
		maxid2 = dateToId.run(date2)
	except:
		logging.error("error in date format, make sure no following zeroes example: 2020/7/15")
		sys.exit(1)

	end_cursor = maxid1 if restore_cursor is None else restore_cursor
	logging.debug(f"end_cursor: {end_cursor}")

	pbar = tqdm(unit='post')
	while end_cursor and end_cursor > maxid2:
		data = pull_json(location, end_cursor)
		if data:
			filename = f"{PATH}/{location}_{date1.replace('/', '-')}_{date2.replace('/', '-')}"
			save_jsonl(data['edges'], filename + '.jsonl')
			end_cursor = data['page_info']['end_cursor']
			open(f"{filename}_CURSOR.txt", "w").write(str(end_cursor))
			pbar.update(len(data['edges']))
		else:
			logging.warning(f"No data found at end_cursor: {end_cursor}")
	else:
		pbar.close()


def main():
	parser = argparse.ArgumentParser()
	parser.add_argument("--dir", nargs=1, default=['./data'])
	parser.add_argument("--max", nargs=1, required=True)
	parser.add_argument("--min", nargs=1, required=True)
	parser.add_argument("--location", nargs=1, required=True)
	parser.add_argument('--restore-cursor', action='store_true', default=False)
	parser.add_argument('--log-level', type=int, default=20)

	args = parser.parse_args()
	global METRO
	global PATH
	global log
	PATH = args.dir[0]
	METRO = json.load(open("./metros.json", 'r'))
	logging.basicConfig(level=args.log_level)
	log = logging.getLogger(__name__)

	assert args.location[0] in METRO, 'Location not available in metros.json'

	if args.restore_cursor:
		logging.warning(f"--max {args.max[0]} will be ignored. Scraping as far back as {args.min[0]}")
		filename = f"{PATH}/{args.location[0]}_{args.max[0].replace('/', '-')}_{args.min[0].replace('/', '-')}"
		maxid1 = open(f"{filename}_CURSOR.txt", "r").read()
	else:
		maxid1 = None
		logging.info(f"Scraping between dates {args.min[0]} and {args.max[0]}")

	logging.info(f"PATH: {PATH}")
	scrape(args.max[0], args.min[0], args.location[0], maxid1)


def test():
	global METRO
	global PATH
	global log
	PATH = "./data"
	METRO = json.load(open("./metros.json", 'r'))
	logging.basicConfig(level=10)
	log = logging.getLogger(__name__)
	scrape("2020/07/16", "2020/07/15", "Memphis")


if __name__ == "__main__":
	main()
# 